function example_optimization

close all
clear all
clc
% Example parameter optimization. As an example: we want to find the
%parameter value k_bp that best describe the recovery of the epithelial barrier after injury.
% The data we have comprises 4 datapoints. 

%I wrote this example inside a function, so that all
% the necessary m-files are in the same place.

%% Experimental data
% Input experimental data (if you have large amounts of data, it might be
% better to store it in a sepparate .mat file and load it here, using the
% command load('name_of_data_file.mat').

t_exp=[ 0     4    12    24]; % Time of measurement, in hours
y_exp=[0.1000    0.2000    0.7000    0.8000]; % epithelial barrier strength (normalized to its maximal value)


%% Mathematical model (its the same as the one inside the CostFunction)
B_t_function=@(B_0, k_bp, t)  1+  exp(-k_bp*t)*(B_0 -1);

%% Optimization

% Give an initial guess for the parameter
xinit= 0.05;
% Set some options
options= optimset('Algorithm','active-set','TolFun',1e-4,'TolX',1e-6,'MaxIter',1e4,'MaxFunEvals',1e6);

% Run the optimization!
[x,J,flag]=fminsearch(@(x)CostFunction(x,t_exp, y_exp),xinit,options);


%% Visualize if the optimization is ok
%Run the model with the parameter we found
t_simulation=t_exp(1):0.1:t_exp(end)+1;
B_0=t_exp(1);
k_bp=x;
B_t_optimized=B_t_function(B_0, k_bp, t_simulation) ;

figure();
plot(t_simulation,B_t_optimized, 'b',t_exp, y_exp, 'bo','MarkerSize', 10,'LineWidth',2)
xlabel('Time [h]', 'FontSize', 12);
ylabel('Barrier','FontSize', 12);
title(['fitting \kappa_B_P=' num2str(x)],'FontSize', 12);
set(gcf, 'Position', [100 100 500 500]); 
axis square
xlim([0, t_simulation(end)]);
ylim([0, 1]);


end


function Out=CostFunction(k_bp,t_exp, y_exp)

% This is the equation of the model (we have in this case the solution for
% the ODE, B(t), hence no need to simulate Bdot).
B_t_function=@(B_0, k_bp, t)  1+  exp(-k_bp*t)*(B_0 -1);

% Simulate the model
y_predicted=B_t_function(t_exp(1),k_bp, t_exp);

% Calculate the cost of the predition vs. the experimental data
Out=sum((y_predicted-y_exp ).^2); %squared error

end
